@@ -29,7 +29,7 @@ freely, subject to the following restrictions:
 #endif
 
 #if defined(_WIN32) && defined(BUILD_DLL)
-# define CGIODLL _declspec(dllexport)
+# define CGIODLL __declspec(dllexport)
 #else
 # define CGIODLL
 #endif
@@ -434,7 +434,7 @@ CGIODLL void cgio_children_ids_f(
 
 /*---------------------------------------------------------*/
 
-CGIODLL void FMNAME(cgio_children_names_f, cgio_children_names_f) (
+CGIODLL void FMNAME(cgio_children_names_f, CGIO_CHILDREN_NAMES_F) (
     cgint_f *cgio_num, double *pid, cgint_f *start, cgint_f *max_ret,
     cgint_f *name_len, cgint_f *num_ret, STR_PSTR(names),
     cgint_f *ier STR_PLEN(names))
@@ -529,18 +529,6 @@ CGIODLL void cgio_get_data_size_f(
 
 /*---------------------------------------------------------*/
 
-CGIODLL void FMNAME(cgio_get_dimensions_f_c, CGIO_GET_DIMENSIONS_F_C) (
-    cgint_f *cgio_num, double *id, cgint_f *ndims, cgsize_t *dims,
-    cgint_f *ier)
-{
-    int i_ndims;
-
-    *ier = (cgint_f)cgio_get_dimensions((int)*cgio_num, *id, &i_ndims, dims);
-    *ndims = (cgint_f)i_ndims;
-}
-
-/*---------------------------------------------------------*/
-
 CGIODLL void FMNAME(cgio_read_all_data_f, CGIO_READ_ALL_DATA_F) (
     cgint_f *cgio_num, double *id, void *data, cgint_f *ier)
 {
@@ -595,19 +583,6 @@ CGIODLL void FMNAME(cgio_set_label_f, CGIO_SET_LABEL_F) (
 
 /*---------------------------------------------------------*/
 
-CGIODLL void FMNAME(cgio_set_dimensions_f_c, CGIO_SET_DIMENSIONS_F_C) (
-    cgint_f *cgio_num, double *id, STR_PSTR(data_type), cgint_f *ndims,
-    cgsize_t *dims, cgint_f *ier STR_PLEN(data_type))
-{
-    char c_type[CGIO_MAX_DATATYPE_LENGTH+1];
-
-    to_c_string(STR_PTR(data_type), STR_LEN(data_type),
-        c_type, CGIO_MAX_DATATYPE_LENGTH);
-    *ier = (cgint_f)cgio_set_dimensions((int)*cgio_num, *id, c_type, (int)*ndims, dims);
-}
-
-/*---------------------------------------------------------*/
-
 CGIODLL void FMNAME(cgio_write_all_data_f, CGIO_WRITE_ALL_DATA_F) (
     cgint_f *cgio_num, double *id, void *data, cgint_f *ier)
 {
