@@ -46,10 +46,25 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
-
+#include <stdint.h>
+#include <limits.h>
 #include "pcgnslib.h"
 #include "mpi.h"
 
+#if SIZE_MAX == UCHAR_MAX
+   #define MPI_SIZE_T MPI_UNSIGNED_CHAR
+#elif SIZE_MAX == USHRT_MAX
+   #define MPI_SIZE_T MPI_UNSIGNED_SHORT
+#elif SIZE_MAX == UINT_MAX
+   #define MPI_SIZE_T MPI_UNSIGNED
+#elif SIZE_MAX == ULONG_MAX
+   #define MPI_SIZE_T MPI_UNSIGNED_LONG
+#elif SIZE_MAX == ULLONG_MAX
+   #define MPI_SIZE_T MPI_UNSIGNED_LONG_LONG
+#else
+   #error "size_t size not found"
+#endif
+
 #define false 0
 #define true 1
 
@@ -57,8 +72,10 @@ int comm_size;
 int comm_rank;
 MPI_Info info;
 
+
+int piomode = CGP_COLLECTIVE; /* DEFAULT */ 
 /* cgsize_t Nelem = 33554432; */
-cgsize_t Nelem = 65536;
+cgsize_t Nelem = 65536; /* DEFAULT */
 cgsize_t NodePerElem = 6;
 
 cgsize_t Nnodes;
@@ -95,43 +112,65 @@ int  debug;
 double t0, t1, t2;
 
 /*
- * Timing storage convention:
- * timing(0) = Total program time, 2,3,4
- * timing(1) = Time to write nodal coordinates, 5,6,7
- * timing(2) = Time to write connectivity table, 8,9,10
- * timing(3) = Time to write solution data (field data), 11,12,13
- * timing(4) = Time to write array data, 14,15,16
- * timing(5) = Time to read nodal coordinates, 17,18,19
- * timing(6) = Time to read connectivity table, 20,21,22
- * timing(7) = Time to read solution data (field data), 23,24,25
- * timing(8) = Time to read array data, 26,27,28
- * timing(9) = Time for cgp_open, CG_MODE_WRITE, 29,30,31
- * timing(10) = Time for cg_base_write, cg_zone_write, 32,33,34
- * timing(11) = Time for cgp_open, CG_MODE_READ, 35,36,37
- * timing(12) = Time for cg_read base, zone, 38,39,40
- * timing(13) = Time for cg_close, WRITE, 41,42,43
- * timing(14) = Time for cg_close, READ, 44,45,46
+ * Timing storage convention:                            avg.| min. | max.
+ * timing(0) = Total program time,                        2,    3,    4
+ * timing(1) = Time to write nodal coordinates            5,    6,    7
+ * timing(2) = Time to write connectivity table           8,    9,    10
+ * timing(3) = Time to write solution data (field data)  11,   12,    13
+ * timing(4) = Time to write array data                  14,   15,    16
+ * timing(5) = Time to read nodal coordinates            17,   18,    19
+ * timing(6) = Time to read connectivity table,          20,   21,    22
+ * timing(7) = Time to read solution data (field data)   23,   24,    25
+ * timing(8) = Time to read array data                   26,   27,    28
+ * timing(9) = Time for cgp_open, CG_MODE_WRITE          29,   30,    31
+ * timing(10) = Time for cgp_open, CG_MODE_READ          32,   33,    34
+ * timing(11) = Time for cg_close, WRITE                 35,   36,    37
+ * timing(12) = Time for cg_close, READ                  38,   39,    40
+ * timing(13) = Time for cg_base_write, cg_zone_write    41,   42,    43
+ * timing(14) = Time for cg_read base, cg_zone_read      44,   45,    46
  */
 double xtiming[15], timing[15], timingMin[15], timingMax[15];
 
-int piomode[2] = {0, 1};
-int piomode_i;
+int read_inputs(int* argc, char*** argv) {
+  int k;
+
+  if(comm_rank==0) {
+    for(k=1;k<*argc;k++) {
+      if(strcmp((*argv)[k],"-nelem")==0) {
+        k++;
+        sscanf((*argv)[k],"%zu",&Nelem);
+      }
+      if(strcmp((*argv)[k],"-ind")==0) {
+        piomode = CGP_INDEPENDENT; 
+      }
+
+    }
+  }
+  MPI_Bcast(&Nelem, 1, MPI_SIZE_T, 0, MPI_COMM_WORLD);
+  MPI_Bcast(&piomode, 1, MPI_INT, 0, MPI_COMM_WORLD);
+
+  return 0;
+}
 
 int initialize(int* argc, char** argv[]) {
-	MPI_Init(argc,argv);
-	MPI_Comm_size(MPI_COMM_WORLD, &comm_size);
-	MPI_Comm_rank(MPI_COMM_WORLD, &comm_rank);
-	MPI_Info_create(&info);
-	MPI_Info_set(info, "striping_unit", "8388608");
-	/* or whatever your GPFS block size actually is*/
-	return 0;
+  MPI_Init(argc,argv);
+  MPI_Comm_size(MPI_COMM_WORLD, &comm_size);
+  MPI_Comm_rank(MPI_COMM_WORLD, &comm_rank);
+  MPI_Info_create(&info);
+  MPI_Info_set(info, "striping_unit", "8388608");
+  /* or whatever your GPFS block size actually is*/
+
+  if(*argc > 2) 
+    read_inputs(argc,argv);
+  
+  return 0;
 }
 
 int c_double_eq(double a, double b) {
 
   double eps = 1.e-8;
 
-  if(a-b < eps) {
+  if(fabs(a-b) < eps) {
     return true;
   }
   return false;
@@ -150,14 +189,15 @@ int main(int argc, char* argv[]) {
 
   size_t Mb_coor, Mb_elem, Mb_field, Mb_array;
 
+  const char* PIOMODE[] = {"IND", "COLL"};
+
   /* parameters */
-  piomode_i = 1;
   debug = false;
 
   t0 = MPI_Wtime(); /* Timer */
 
   err = (int)cgp_mpi_info(info);
-  err = (int)cgp_pio_mode((CGNS_ENUMT(PIOmode_t))piomode_i);
+  err = (int)cgp_pio_mode((CGNS_ENUMT(PIOmode_t))piomode);
 
   Nnodes = Nelem*NodePerElem;
 
@@ -191,11 +231,11 @@ int main(int argc, char* argv[]) {
     printf("*FAILED* cg_base_write \n");
     cgp_error_exit();
   }
-  if(cg_zone_write(fn, B, "Zone 1", nijk, Unstructured, &Z) != CG_OK) {
+  if(cg_zone_write(fn, B, "Zone 1", nijk, CGNS_ENUMV(Unstructured), &Z) != CG_OK) {
     printf("*FAILED* cg_zone_write \n");
     cgp_error_exit();
   t2 = MPI_Wtime();
-  xtiming[10] = t2-t1;
+  xtiming[13] = t2-t1;
 
   }
 
@@ -278,7 +318,7 @@ int main(int argc, char* argv[]) {
   start = 1;
   end = nijk[1];
 
-  if(cgp_section_write(fn,B,Z,"Elements",PENTA_6,start,end,0,&S) != CG_OK) {
+  if(cgp_section_write(fn,B,Z,"Elements",CGNS_ENUMV(PENTA_6),start,end,0,&S) != CG_OK) {
     printf("*FAILED* cgp_section_write \n");
     cgp_error_exit();
   }
@@ -337,7 +377,7 @@ int main(int argc, char* argv[]) {
      Data_Fz[k] = comm_rank*count+k + 1.03;
   }
 
-  if(cg_sol_write(fn, B, Z, "Solution", Vertex, &S) != CG_OK) {
+  if(cg_sol_write(fn, B, Z, "Solution", CGNS_ENUMV(Vertex), &S) != CG_OK) {
     printf("*FAILED* cg_sol_write \n");
     cgp_error_exit();
   }
@@ -476,7 +516,7 @@ int main(int argc, char* argv[]) {
     cgp_error_exit();
   };
   t2 = MPI_Wtime();
-  xtiming[13] = t2-t1;
+  xtiming[11] = t2-t1;
 
   /* ====================================== */
   /* ==    **  READ THE CGNS FILE **     == */
@@ -490,7 +530,7 @@ int main(int argc, char* argv[]) {
     cgp_error_exit();
   }
   t2 = MPI_Wtime();
-  xtiming[11] = t2-t1;
+  xtiming[10] = t2-t1;
 
   /* Read the base information */
   t1 = MPI_Wtime();
@@ -515,7 +555,7 @@ int main(int argc, char* argv[]) {
     cgp_error_exit();
   }
   t2 = MPI_Wtime();
-  xtiming[12] = t2-t1;
+  xtiming[14] = t2-t1;
 
   /* Check the read zone information is correct */
   if(sizes[0] != Nnodes) {
@@ -764,7 +804,7 @@ int main(int argc, char* argv[]) {
      cgp_error_exit();
   }
   t2 = MPI_Wtime();
-  xtiming[14] = t2-t1;
+  xtiming[12] = t2-t1;
 
   xtiming[0] = t2-t0;
 
@@ -773,15 +813,16 @@ int main(int argc, char* argv[]) {
   MPI_Reduce(&xtiming, &timingMax, 15, MPI_DOUBLE, MPI_MAX, 0, MPI_COMM_WORLD);
 
   if(comm_rank==0) {
-    sprintf(fname, "timing_%06d_%d.dat", comm_size, piomode_i+1);
+    sprintf(fname, "timing_%06d_%s.dat", comm_size, PIOMODE[piomode]);
     FILE *fid = fopen(fname, "w");
     if (fid == NULL) {
       printf("Error opening timing file!\n");
     } else {
+      fprintf(fid,"#nelem = %zu \n",Nelem);
       fprintf(fid,"#nprocs, total time, write: coord., elem., field, array, read: coord., elem., field, array, MB: coord, elem, field, array \n%d", comm_size);
 
       for ( k = 0; k < 15; k++) {
-	fprintf(fid," %20f %20f %20f ",timing[k]/((double) comm_size), timingMin[k], timingMax[k]);
+	fprintf(fid," %.3f %.3f %.3f ",timing[k]/((double) comm_size), timingMin[k], timingMax[k]);
       }
       fprintf(fid," %zu %zu %zu %zu \n", Mb_coor, Mb_elem, Mb_field, Mb_array);
       fclose(fid);
