@@ -47,29 +47,36 @@ static cgsize_t hexa[27]  = {1, 2, 3, 4, 5, 6, 7, 8, 12, 13, 14, 15,
                              26, 27, 28, 29, 30};
 
 static int npoly = 16;
-static cgsize_t poly[] = {4, 1, 4, 3, 2,
-                          4, 2, 3, 7, 6,
-                          4, 3, 4, 8, 7,
-                          4, 1, 5, 8, 4,
-                          4, 1, 5, 6, 2,
-                          4, 5, 6, 7, 8,
-                          3, 3, 4, 9,
-                          3, 8, 7, 10,
-                          4, 3, 9, 10, 7,
-                          4, 4, 8, 10, 9,
-                          3, 5, 6, 11,
-                          3, 6, 7, 11,
-                          3, 7, 8, 11,
-                          3, 5, 11, 8,
-                          3, 7, 10, 11,
-                          3, 8, 11, 10};
+static cgsize_t poly[] = {1, 4, 3, 2,
+                          2, 3, 7, 6,
+                          3, 4, 8, 7,
+                          1, 5, 8, 4,
+                          1, 5, 6, 2,
+                          5, 6, 7, 8,
+                          3, 4, 9,
+                          8, 7, 10,
+                          3, 9, 10, 7,
+                          4, 8, 10, 9,
+                          5, 6, 11,
+                          6, 7, 11,
+                          7, 8, 11,
+                          5, 11, 8,
+                          7, 10, 11,
+                          8, 11, 10};
+static cgsize_t poly_offsets[] = {0, 4, 8, 12, 16,
+                                  20, 24, 27,
+                                  30, 34, 38, 41,
+                                  44, 47, 50, 53, 56};
+
 static int nface = 4;
-static cgsize_t face[] = {6, 1, 2, 3, 4, 5, 6,
-                          5, -3, 7, 8, 9, 10,
-                          5, -6, 11, 12, 13, 14,
-                          4, -8, -13, 15, 16};
+static cgsize_t face[] = {1, 2, 3, 4, 5, 6,
+                          -3, 7, 8, 9, 10,
+                          -6, 11, 12, 13, 14,
+                          -8, -13, 15, 16};
+static cgsize_t face_offsets[] = {0, 6, 11, 16, 20};
 
 static cgsize_t elems[256];
+static cgsize_t elems_offsets[256];
 
 /* cubic elements */
 
@@ -155,8 +162,8 @@ int main (int argc, char *argv[])
 
     /* NGON_n first so polyhedra face references are correct */
 
-    if (cg_section_write (fnum, bnum, znum, "NGON_n", CGNS_ENUMV(NGON_n),
-            ne+1, ne+npoly, 0, poly, &snum))
+    if (cg_poly_section_write (fnum, bnum, znum, "NGON_n", CGNS_ENUMV(NGON_n),
+            ne+1, ne+npoly, 0, poly, poly_offsets, &snum))
         cg_error_exit();
     ne += npoly;
 
@@ -167,8 +174,10 @@ int main (int argc, char *argv[])
         cg_error_exit ();
     ne++;
 
+    elems_offsets[0] = 0;
     elems[j++] = (int)CGNS_ENUMV(NODE);
     elems[j++] = node[0];
+    elems_offsets[ne-npoly] = j;
 
     /* BAR_2 */
 
@@ -180,6 +189,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(BAR_2);
     for (n = 0; n < 2; n++)
         elems[j++] = bar[n];
+    elems_offsets[ne-npoly] = j;
 
     /* TRI_3 */
 
@@ -187,10 +197,12 @@ int main (int argc, char *argv[])
             ne+1, ne+1, 0, tri, &snum))
         cg_error_exit ();
     ne++;
+    elems_offsets[ne-npoly] = j;
 
     elems[j++] = (int)CGNS_ENUMV(TRI_3);
     for (n = 0; n < 3; n++)
         elems[j++] = tri[n];
+    elems_offsets[ne-npoly] = j;
 
     /* QUAD_4 */
 
@@ -202,6 +214,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(QUAD_4);
     for (n = 0; n < 4; n++)
         elems[j++] = quad9[n];
+    elems_offsets[ne-npoly] = j;
 
     /* TETRA_4 */
 
@@ -213,6 +226,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(TETRA_4);
     for (n = 0; n < 4; n++)
         elems[j++] = tetra[n];
+    elems_offsets[ne-npoly] = j;
 
     /* PYRA_5 */
 
@@ -224,6 +238,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(PYRA_5);
     for (n = 0; n < 5; n++)
         elems[j++] = pyra[n];
+    elems_offsets[ne-npoly] = j;
 
     /* PENTA_6 */
 
@@ -235,6 +250,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(PENTA_6);
     for (n = 0; n < 6; n++)
         elems[j++] = penta[n];
+    elems_offsets[ne-npoly] = j;
 
     /* HEXA_8 */
 
@@ -246,18 +262,19 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(HEXA_8);
     for (n = 0; n < 8; n++)
         elems[j++] = hexa[n];
+    elems_offsets[ne-npoly] = j;
 
     /* MIXED */
 
-    if (cg_section_write (fnum, bnum, znum, "MIXED", CGNS_ENUMV(MIXED),
-            ne+1, ne+8, 0, elems, &snum))
+    if (cg_poly_section_write (fnum, bnum, znum, "MIXED", CGNS_ENUMV(MIXED),
+            ne+1, ne+8, 0, elems, elems_offsets, &snum))
         cg_error_exit ();
     ne += 8;
 
     /* NFACE_n */
 
-    if (cg_section_write (fnum, bnum, znum, "NFACE_n", CGNS_ENUMV(NFACE_n),
-            ne+1, ne+nface, 0, face, &snum))
+    if (cg_poly_section_write (fnum, bnum, znum, "NFACE_n", CGNS_ENUMV(NFACE_n),
+            ne+1, ne+nface, 0, face, face_offsets, &snum))
         cg_error_exit ();
 
     /* zone with quadratic elements */
@@ -290,9 +307,11 @@ int main (int argc, char *argv[])
         cg_error_exit ();
     ne++;
 
+    elems_offsets[0] = 0;
     elems[j++] = (int)CGNS_ENUMV(BAR_3);
     for (n = 0; n < 3; n++)
         elems[j++] = bar[n];
+    elems_offsets[ne] = j;
 
     /* TRI_6 */
 
@@ -304,6 +323,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(TRI_6);
     for (n = 0; n < 6; n++)
         elems[j++] = tri[n];
+    elems_offsets[ne] = j;
 
     /* QUAD_8 */
 
@@ -315,6 +335,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(QUAD_8);
     for (n = 0; n < 8; n++)
         elems[j++] = quad9[n];
+    elems_offsets[ne] = j;
 
     /* TETRA_10 */
 
@@ -326,6 +347,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(TETRA_10);
     for (n = 0; n < 10; n++)
         elems[j++] = tetra[n];
+    elems_offsets[ne] = j;
 
     /* PYRA_13 */
 
@@ -337,6 +359,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(PYRA_13);
     for (n = 0; n < 13; n++)
         elems[j++] = pyra[n];
+    elems_offsets[ne] = j;
 
     /* PENTA_15 */
 
@@ -348,6 +371,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(PENTA_15);
     for (n = 0; n < 15; n++)
         elems[j++] = penta[n];
+    elems_offsets[ne] = j;
 
     /* HEXA_20 */
 
@@ -359,11 +383,12 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(HEXA_20);
     for (n = 0; n < 20; n++)
         elems[j++] = hexa[n];
+    elems_offsets[ne] = j;
 
     /* MIXED */
 
-    if (cg_section_write (fnum, bnum, znum, "MIXED", CGNS_ENUMV(MIXED),
-            ne+1, ne+7, 0, elems, &snum))
+    if (cg_poly_section_write (fnum, bnum, znum, "MIXED", CGNS_ENUMV(MIXED),
+            ne+1, ne+7, 0, elems, elems_offsets, &snum))
         cg_error_exit ();
     ne += 7;
 
@@ -397,9 +422,11 @@ int main (int argc, char *argv[])
         cg_error_exit ();
     ne++;
 
+    elems_offsets[0] = 0;
     elems[j++] = (int)CGNS_ENUMV(QUAD_9);
     for (n = 0; n < 9; n++)
         elems[j++] = quad9[n];
+    elems_offsets[ne] = j;
 
     /* TETRA_10 */
 
@@ -411,6 +438,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(TETRA_10);
     for (n = 0; n < 10; n++)
         elems[j++] = tetra[n];
+    elems_offsets[ne] = j;
 
     /* PYRA_14 */
 
@@ -422,6 +450,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(PYRA_14);
     for (n = 0; n < 14; n++)
         elems[j++] = pyra[n];
+    elems_offsets[ne] = j;
 
     /* PENTA_18 */
 
@@ -433,6 +462,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(PENTA_18);
     for (n = 0; n < 18; n++)
         elems[j++] = penta[n];
+    elems_offsets[ne] = j;
 
     /* HEXA_27 */
 
@@ -444,11 +474,12 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(HEXA_27);
     for (n = 0; n < 27; n++)
         elems[j++] = hexa[n];
+    elems_offsets[ne] = j;
 
     /* MIXED */
 
-    if (cg_section_write (fnum, bnum, znum, "MIXED", CGNS_ENUMV(MIXED),
-            ne+1, ne+5, 0, elems, &snum))
+    if (cg_poly_section_write (fnum, bnum, znum, "MIXED", CGNS_ENUMV(MIXED),
+            ne+1, ne+5, 0, elems, elems_offsets, &snum))
         cg_error_exit ();
     ne += 5;
 
@@ -482,9 +513,11 @@ int main (int argc, char *argv[])
         cg_error_exit ();
     ne++;
 
+    elems_offsets[0] = 0;
     elems[j++] = (int)CGNS_ENUMV(BAR_4);
     for (n = 0; n < 4; n++)
         elems[j++] = bar4[n];
+    elems_offsets[ne] = j;
 
     /* TRI_9 */
 
@@ -496,6 +529,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(TRI_9);
     for (n = 0; n < 9; n++)
         elems[j++] = tri9[n];
+    elems_offsets[ne] = j;
 
     /* QUAD_12 */
 
@@ -507,6 +541,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(QUAD_12);
     for (n = 0; n < 12; n++)
         elems[j++] = quad12[n];
+    elems_offsets[ne] = j;
 
     /* TETRA_16 */
 
@@ -518,6 +553,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(TETRA_16);
     for (n = 0; n < 16; n++)
         elems[j++] = tetra16[n];
+    elems_offsets[ne] = j;
 
     /* PYRA_21 */
 
@@ -529,6 +565,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(PYRA_21);
     for (n = 0; n < 21; n++)
         elems[j++] = pyra21[n];
+    elems_offsets[ne] = j;
 
     /* PENTA_24 */
 
@@ -540,6 +577,7 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(PENTA_24);
     for (n = 0; n < 24; n++)
         elems[j++] = penta24[n];
+    elems_offsets[ne] = j;
 
     /* HEXA_32 */
 
@@ -551,11 +589,12 @@ int main (int argc, char *argv[])
     elems[j++] = (int)CGNS_ENUMV(HEXA_32);
     for (n = 0; n < 32; n++)
         elems[j++] = hexa32[n];
+    elems_offsets[ne] = j;
 
     /* MIXED */
 
-    if (cg_section_write (fnum, bnum, znum, "MIXED", CGNS_ENUMV(MIXED),
-            ne+1, ne+7, 0, elems, &snum))
+    if (cg_poly_section_write (fnum, bnum, znum, "MIXED", CGNS_ENUMV(MIXED),
+            ne+1, ne+7, 0, elems, elems_offsets, &snum))
         cg_error_exit ();
     ne += 7;
 
